// Main entry point for random map script
void main(void)
{
rmCreateTrigger("night");
   rmCreateTrigger("Day");

   rmSwitchToTrigger(rmTriggerID("night"));
   rmSetTriggerActive(true);
   rmAddTriggerCondition("Timer");
   rmSetTriggerConditionParamInt("Param1", 300);
   rmAddTriggerEffect("Set Lighting");
   rmSetTriggerEffectParam("SetName", "night");
   rmSetTriggerEffectParamInt("FadeTime", 30);
   rmAddTriggerEffect("Fire Event");
   rmSetTriggerEffectParamInt("EventID", rmTriggerID("Day"));

   rmSwitchToTrigger(rmTriggerID("Day"));
   rmSetTriggerActive(false);
   rmAddTriggerCondition("Timer");
   rmSetTriggerConditionParamInt("Param1", 60);
   rmAddTriggerEffect("Set Lighting");
   rmSetTriggerEffectParam("SetName", "Default");
   rmSetTriggerEffectParamInt("FadeTime", 30);
   rmAddTriggerEffect("Fire Event");
   rmSetTriggerEffectParamInt("EventID", rmTriggerID("night"));

   // Set size.
   int playerTiles=12800;
   int size=3.0*sqrt(cNumberNonGaiaPlayers*playerTiles);
   rmEchoInfo("Map size="+size+"m x "+size+"m");
   rmSetMapSize(size, size);

   // Set up default water.
   rmSetSeaLevel(0.0);
   rmSetSeaType("egyptian nile");
   // Init map.
   rmTerrainInitialize("water");

   // Define some classes.
   int classPlayer=rmDefineClass("player");
   int classBonusIsland=rmDefineClass("bonus island");
   rmDefineClass("corner");
   rmDefineClass("starting settlement");

   // -------------Define constraints
   
   // Create a edge of map constraint.
   int playerEdgeConstraint=rmCreateBoxConstraint("player edge of map", rmXTilesToFraction(-1), rmZTilesToFraction(-1), 1.0-rmXTilesToFraction(-1), 1.0-rmZTilesToFraction(-1), 0.01);

   // Player area constraint.
   int playerConstraint=rmCreateClassDistanceConstraint("stay away from players", classPlayer, 15.0);
   int shortPlayerConstraint=rmCreateClassDistanceConstraint("short stay away from players", classPlayer, 2.0);

   // Bonus area constraint.
   int bonusIslandConstraint=rmCreateClassDistanceConstraint("avoid bonus island", classBonusIsland, 30.0);
   int playerIslandsConstraint=rmCreateClassDistanceConstraint("avoid player", classPlayer, 35.0);

   // corner constraint.
   int cornerConstraint=rmCreateClassDistanceConstraint("stay away from corner", rmClassID("corner"), 15.0);
   int cornerOverlapConstraint=rmCreateClassDistanceConstraint("don't overlap corner", rmClassID("corner"), 2.0);

   // Settlement constraint.
   int avoidSettlement=rmCreateTypeDistanceConstraint("avoid settlement", "AbstractSettlement", 40.0);
   int shortAvoidSettlement=rmCreateTypeDistanceConstraint("short avoid settlement", "AbstractSettlement", 10.0);
   int farAvoidSettlement=rmCreateTypeDistanceConstraint("TCs avoid TCs by long distance", "AbstractSettlement", 20.0);

   // Far starting settlement constraint.
   int farStartingSettleConstraint=rmCreateClassDistanceConstraint("far start settle", rmClassID("starting settlement"), 70.0);

   // Tower constraint.
   int avoidTower=rmCreateTypeDistanceConstraint("avoid tower", "tower", 25.0);

   // Gold
   int avoidGold=rmCreateTypeDistanceConstraint("avoid gold", "gold", 30.0);
   int shortAvoidGold=rmCreateTypeDistanceConstraint("short avoid gold", "gold", 10.0);

   // Goats/pigs
   int avoidHerdable=rmCreateTypeDistanceConstraint("avoid herdable", "herdable", 20.0);

   // Animals
   int classBonusHuntable=rmDefineClass("bonus huntable");
   int avoidHuntable=rmCreateClassDistanceConstraint("avoid bonus huntable", classBonusHuntable, 15.0);
   int avoidFood=rmCreateTypeDistanceConstraint("avoid other food sources", "food", 6.0);
   int avoidPredator=rmCreateTypeDistanceConstraint("avoid predator", "animalPredator", 20.0);

   // Avoid impassable land
   int avoidImpassableLand=rmCreateTerrainDistanceConstraint("avoid impassable land", "Land", false, 10.0);
   int shortAvoidImpassableLand=rmCreateTerrainDistanceConstraint("short avoid impassable land", "Land", false, 5.0);

   // -------------Define objects
   // Close Objects

   int startingSettlementID=rmCreateObjectDef("Starting settlement");
   rmAddObjectDefItem(startingSettlementID, "Settlement Level 1", 1, 0.0);
   rmAddObjectDefToClass(startingSettlementID, rmClassID("starting settlement"));
   rmSetObjectDefMinDistance(startingSettlementID, 0.0);
   rmSetObjectDefMaxDistance(startingSettlementID, 0.0);

   // towers avoid other towers
   int startingTowerID=rmCreateObjectDef("Starting tower");
   rmAddObjectDefItem(startingTowerID, "tower", 1, 0.0);
   rmSetObjectDefMinDistance(startingTowerID, 22.0);
   rmSetObjectDefMaxDistance(startingTowerID, 25.0);
   rmAddObjectDefConstraint(startingTowerID, avoidTower);
   rmAddObjectDefConstraint(startingTowerID, avoidImpassableLand);

   int startingGoldID=rmCreateObjectDef("Starting gold");
   rmAddObjectDefItem(startingGoldID, "Gold mine small", 1, 0.0);
   rmSetObjectDefMinDistance(startingGoldID, 34.0);
   rmSetObjectDefMaxDistance(startingGoldID, 40.0);
   rmAddObjectDefConstraint(startingGoldID, avoidGold);
   rmAddObjectDefConstraint(startingGoldID, avoidImpassableLand);

   // pigs
   int closePigsID=rmCreateObjectDef("close pigs");
   rmAddObjectDefItem(closePigsID, "pig", 2, 2.0);
   rmSetObjectDefMinDistance(closePigsID, 20.0);
   rmSetObjectDefMaxDistance(closePigsID, 25.0);
   rmAddObjectDefConstraint(closePigsID, avoidFood);   

   int closeChickensID=rmCreateObjectDef("close Chickens");
   rmAddObjectDefItem(closeChickensID, "chicken", 8, 5.0);
   rmSetObjectDefMinDistance(closeChickensID, 20.0);
   rmSetObjectDefMaxDistance(closeChickensID, 25.0);
   rmAddObjectDefConstraint(closeChickensID, avoidFood); 

   int closeBerriesID=rmCreateObjectDef("close berries");
   rmAddObjectDefItem(closeBerriesID, "berry bush", 6, 4.0);
   rmSetObjectDefMinDistance(closeBerriesID, 20.0);
   rmSetObjectDefMaxDistance(closeBerriesID, 25.0);
   rmAddObjectDefConstraint(closeBerriesID, avoidFood);   
   
   int closeBoarID=rmCreateObjectDef("close Boar");
   float boarNumber=rmRandFloat(0, 1);
   if(boarNumber<0.3)
      rmAddObjectDefItem(closeBoarID, "boar", 1, 4.0);
   else if(boarNumber<0.6)
      rmAddObjectDefItem(closeBoarID, "boar", 2, 4.0);
   else 
      rmAddObjectDefItem(closeBoarID, "boar", 3, 6.0);
   rmSetObjectDefMinDistance(closeBoarID, 30.0);
   rmSetObjectDefMaxDistance(closeBoarID, 50.0);

   int stragglerTreeID=rmCreateObjectDef("straggler tree");
   rmAddObjectDefItem(stragglerTreeID, "palm", 1, 0.0);
   rmSetObjectDefMinDistance(stragglerTreeID, 12.0);
   rmSetObjectDefMaxDistance(stragglerTreeID, 15.0);

   // Medium Objects

   // gold avoids gold and Settlements
   int mediumGoldID=rmCreateObjectDef("medium gold");
   rmAddObjectDefItem(mediumGoldID, "Gold mine", 1, 0.0);
   rmSetObjectDefMinDistance(mediumGoldID, 40.0);
   rmSetObjectDefMaxDistance(mediumGoldID, 60.0);
   rmAddObjectDefConstraint(mediumGoldID, avoidGold);
   rmAddObjectDefConstraint(mediumGoldID, shortAvoidSettlement);
   rmAddObjectDefConstraint(mediumGoldID, avoidImpassableLand);

   int mediumPigsID=rmCreateObjectDef("medium pigs");
   rmAddObjectDefItem(mediumPigsID, "pig", 4, 4.0);
   rmSetObjectDefMinDistance(mediumPigsID, 50.0);
   rmSetObjectDefMaxDistance(mediumPigsID, 70.0);
   rmAddObjectDefConstraint(mediumPigsID, shortAvoidImpassableLand);
   
   // Far Objects

   // Settlement avoid Settlements
   int farSettlementID=rmCreateObjectDef("far settlement");
   rmAddObjectDefItem(farSettlementID, "Settlement", 1, 0.0);
   rmSetObjectDefMinDistance(farSettlementID, 80.0);
   rmSetObjectDefMaxDistance(farSettlementID, 150.0);
   rmAddObjectDefConstraint(farSettlementID, avoidImpassableLand);
   rmAddObjectDefConstraint(farSettlementID, farAvoidSettlement);

      
   // gold avoids gold, Settlements and TCs
   int farGoldID=rmCreateObjectDef("far gold");
   rmAddObjectDefItem(farGoldID, "Gold mine", 2, 8.0);
   rmSetObjectDefMinDistance(farGoldID, 150.0);
   rmSetObjectDefMaxDistance(farGoldID, 300.0);
   rmAddObjectDefConstraint(farGoldID, avoidGold);
   rmAddObjectDefConstraint(farGoldID, shortAvoidImpassableLand);
   rmAddObjectDefConstraint(farGoldID, shortAvoidSettlement);

   // gaz avoid gaz
   int fargazelleID=rmCreateObjectDef("far gazelle");
   rmAddObjectDefItem(fargazelleID, "gazelle", 8, 4.0);
   rmSetObjectDefMinDistance(fargazelleID, 80.0);
   rmSetObjectDefMaxDistance(fargazelleID, 150.0);
   rmAddObjectDefConstraint(fargazelleID, avoidHerdable);
   rmAddObjectDefConstraint(fargazelleID, shortAvoidImpassableLand);
   
   // predators
   // avoid TCs
   int farPredatorID=rmCreateObjectDef("far predator");
   float predatorSpecies=rmRandFloat(0, 1);
   if(predatorSpecies<0.5)   
      rmAddObjectDefItem(farPredatorID, "hyena", 2, 8.0);
   else
      rmAddObjectDefItem(farPredatorID, "crocodile", 2, 20.0);
   rmSetObjectDefMinDistance(farPredatorID, 50.0);
   rmSetObjectDefMaxDistance(farPredatorID, 100.0);
   rmAddObjectDefConstraint(farPredatorID, avoidPredator);
   rmAddObjectDefConstraint(farPredatorID, farStartingSettleConstraint);
   rmAddObjectDefConstraint(farPredatorID, shortAvoidImpassableLand);
   
   // This map will either use boar or deer as the extra huntable food.


   int randomTreeID=rmCreateObjectDef("random tree");
   rmAddObjectDefItem(randomTreeID, "palm", 5, 10.0);
   rmSetObjectDefMinDistance(randomTreeID, 0.0);
   rmSetObjectDefMaxDistance(randomTreeID, rmXFractionToMeters(0.5));
   rmAddObjectDefConstraint(randomTreeID, rmCreateTypeDistanceConstraint("random tree", "all", 4.0));
   rmAddObjectDefConstraint(randomTreeID, shortAvoidImpassableLand);
    
   
   // Relics avoid TCs
   int relicID=rmCreateObjectDef("relic");
   rmAddObjectDefItem(relicID, "relic", 1, 0.0);
   rmSetObjectDefMinDistance(relicID, 40.0);
   rmSetObjectDefMaxDistance(relicID, 550.0);
   rmAddObjectDefConstraint(relicID, rmCreateTypeDistanceConstraint("relic vs relic", "relic", 70.0));
   rmAddObjectDefConstraint(relicID, shortAvoidImpassableLand);
   rmAddObjectDefConstraint(relicID, farStartingSettleConstraint);


   // -------------Done defining objects



   

   // Cheesy square placement of players.
   rmPlacePlayersSquare(0.30, 25.0, 10.0); 

    // Set up player areas.
   float playerFraction=rmAreaTilesToFraction(8200);
   for(i=1; <cNumberPlayers)
   {
      // Create the area.
      int id=rmCreateArea("Player"+i);

      // Assign to the player.
      rmSetPlayerArea(i, id);

      // Set the size.
      rmSetAreaSize(id, 0.9*playerFraction, 1.1*playerFraction);

      rmAddAreaToClass(id, classPlayer);

      rmSetAreaMinBlobs(id, 3);
      rmSetAreaMaxBlobs(id, 7);
      rmSetAreaMinBlobDistance(id, 16.0);
      rmSetAreaMaxBlobDistance(id, 40.0);
  /*    rmSetAreaCoherence(id, 0.0); */

      rmSetAreaBaseHeight(id, 2.0);

      rmSetAreaSmoothDistance(id, 10);
      rmSetAreaHeightBlend(id, 3);

      // Add constraints.
      rmAddAreaConstraint(id, playerConstraint); 
      rmAddAreaConstraint(id, cornerOverlapConstraint);
      rmAddAreaConstraint(id, playerEdgeConstraint);

      // Set the location.
      rmSetAreaLocPlayer(id, i);

      // Set type.
      rmSetAreaTerrainType(id, "grassdirt75");
   }

   // Build the areas.
   rmBuildAllAreas();

   // Build up some bonus islands.
   int bonusCount=cNumberNonGaiaPlayers + rmRandInt(1, 3);  // num players plus some extra
   for(i=0; <bonusCount)
   {
      int bonusIslandID=rmCreateArea("bonus island"+i);
      rmSetAreaSize(bonusIslandID, rmAreaTilesToFraction(5500), rmAreaTilesToFraction(6000));
      rmSetAreaTerrainType(bonusIslandID, "grassdirt75");
      rmSetAreaWarnFailure(bonusIslandID, false);
      rmAddAreaConstraint(bonusIslandID, playerEdgeConstraint); 
      rmAddAreaConstraint(bonusIslandID, bonusIslandConstraint);
      rmAddAreaConstraint(bonusIslandID, playerConstraint); 
      rmAddAreaToClass(bonusIslandID, classBonusIsland);

      rmSetAreaCoherence(bonusIslandID, 0.0);

      rmSetAreaSmoothDistance(bonusIslandID, 5);
      rmSetAreaHeightBlend(bonusIslandID, 4);

      rmSetAreaBaseHeight(bonusIslandID, 3.0);

      rmBuildArea(bonusIslandID);
   }

   // Place stuff.

   // TC
   rmPlaceObjectDefPerPlayer(startingSettlementID, true);

   // Settlements.
   rmPlaceObjectDefPerPlayer(farSettlementID, false, 3);

   // Towers.
   rmPlaceObjectDefPerPlayer(startingTowerID, true, 4);

   // Straggler trees.
   rmPlaceObjectDefPerPlayer(stragglerTreeID, false, 3);

   // Gold
   rmPlaceObjectDefPerPlayer(startingGoldID, false);

    // Pigs
   rmPlaceObjectDefPerPlayer(closePigsID, true);

   // Chickens or berries.
   for(i=1; <cNumberPlayers)
   {
      if(rmRandFloat(0.0, 1.0)<0.5)
         rmPlaceObjectDefAtLoc(closeChickensID, 0, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
      else
         rmPlaceObjectDefAtLoc(closeBerriesID, 0, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
   }

   // Boar.
   rmPlaceObjectDefPerPlayer(closeBoarID, false);

   // Medium things....
   // Gold
   rmPlaceObjectDefPerPlayer(mediumGoldID, false);

  
   for(i=1; <cNumberPlayers)
      rmPlaceObjectDefAtLoc(mediumPigsID, 0, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i), 2);
      
   // Far things.
   
   // Gold.
   rmPlaceObjectDefPerPlayer(farGoldID, false, 4);

   // Relics
   rmPlaceObjectDefPerPlayer(relicID, false,4);
   
   // gazelle
   for(i=1; <cNumberPlayers)
      rmPlaceObjectDefAtLoc(fargazelleID, 0, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i), 4);

   // Predators
   rmPlaceObjectDefPerPlayer(farPredatorID, false, 2);

   // Random trees.
   rmPlaceObjectDefAtLoc(randomTreeID, 0, 0.5, 0.5, 30*cNumberNonGaiaPlayers);

   // Random Settlements
   int randSettlementID=rmCreateObjectDef("rand settlement");
   rmAddObjectDefItem(randSettlementID, "settlement", 1, 0.0);
   //rmAddObjectDefConstraint(randSettlementID, avoidImpassableLand);
   rmSetObjectDefMinDistance(randSettlementID, 110.0);
   rmSetObjectDefMaxDistance(randSettlementID, 450.0);
   rmAddObjectDefConstraint(randSettlementID, playerConstraint);
   rmAddObjectDefConstraint(randSettlementID, farAvoidSettlement);

   //Random settlements
   rmPlaceObjectDefPerPlayer(randSettlementID, true, 1);

   // Random Elephants
   int bonusElephantID=rmCreateObjectDef("bonus relics");
   rmAddObjectDefItem(bonusElephantID, "chicken", 4, 3.0);
   rmSetObjectDefMinDistance(bonusElephantID, 110.0);
   rmSetObjectDefMaxDistance(bonusElephantID, 450.0);
   rmAddObjectDefConstraint(bonusElephantID, playerConstraint);

   // Random Zebras
   int bonusZebraID=rmCreateObjectDef("bonus zebra");
   rmAddObjectDefItem(bonusZebraID, "zebra", 6, 8.0);
   rmSetObjectDefMinDistance(bonusZebraID, 110.0);
   rmSetObjectDefMaxDistance(bonusZebraID, 450.0);
   rmAddObjectDefConstraint(bonusZebraID, playerConstraint);

   //Bonus Huntable
   rmPlaceObjectDefPerPlayer(bonusElephantID, false, 1);
   rmPlaceObjectDefPerPlayer(bonusZebraID, false, 1);

 // Birds
   int farhawkID=rmCreateObjectDef("far hawks");
   rmAddObjectDefItem(farhawkID, "hawk", 5, 0.0);
   rmSetObjectDefMinDistance(farhawkID, 0.0);
   rmSetObjectDefMaxDistance(farhawkID, rmXFractionToMeters(0.5));

 //papyrus
   int avoidAll=rmCreateTypeDistanceConstraint("avoid all", "all", 6.0);
   int papyrusID=rmCreateObjectDef("lone papyrus");
   int nearshore=rmCreateTerrainMaxDistanceConstraint("papyrus near shore", "land", true, 4.0);
   rmAddObjectDefItem(papyrusID, "papyrus", 5, 50.0);
   rmSetObjectDefMinDistance(papyrusID, 0.0);
   rmSetObjectDefMaxDistance(papyrusID, 600);
   rmAddObjectDefConstraint(papyrusID, avoidAll);
   rmAddObjectDefConstraint(papyrusID, nearshore);
   rmPlaceObjectDefAtLoc(papyrusID, 0, 0.5, 0.5, 6*cNumberNonGaiaPlayers);

   int papyrus2ID=rmCreateObjectDef("grouped papyrus");
   rmAddObjectDefItem(papyrus2ID, "papyrus", 10, 25.0);
   rmSetObjectDefMinDistance(papyrus2ID, 0.0);
   rmSetObjectDefMaxDistance(papyrus2ID, 600);
   rmAddObjectDefConstraint(papyrus2ID, avoidAll);
   rmAddObjectDefConstraint(papyrus2ID, nearshore);
   rmPlaceObjectDefAtLoc(papyrus2ID, 0, 0.5, 0.5, 2*cNumberNonGaiaPlayers);

   int lilyID=rmCreateObjectDef("pads");
   rmAddObjectDefItem(lilyID, "water lilly", 8, 25.0);
   rmSetObjectDefMinDistance(lilyID, 0.0);
   rmSetObjectDefMaxDistance(lilyID, 600);
   rmAddObjectDefConstraint(lilyID, avoidAll);
   rmPlaceObjectDefAtLoc(lilyID, 0, 0.5, 0.5, 4*cNumberNonGaiaPlayers);

   int lily2ID=rmCreateObjectDef("mo' pads");
   rmAddObjectDefItem(lily2ID, "water lilly", 3, 10.0);
   rmSetObjectDefMinDistance(lily2ID, 0.0);
   rmSetObjectDefMaxDistance(lily2ID, 600);
   rmAddObjectDefConstraint(lily2ID, avoidAll);
   rmPlaceObjectDefAtLoc(lily2ID, 0, 0.5, 0.5, 8*cNumberNonGaiaPlayers);



/*   // Crocs.
   rmPlaceObjectDefInRandomAreaOfClass(farCrocsID, false, classBonusIsland, cNumberNonGaiaPlayers); */

   // Fish
   int fishVsFishID=rmCreateTypeDistanceConstraint("fish v fish", "fish", 10.0);
   int fishID=rmCreateObjectDef("fish");
   rmAddObjectDefItem(fishID, "fish - perch", 1, 2.0);
   rmSetObjectDefMinDistance(fishID, 0.0);
   rmSetObjectDefMaxDistance(fishID, rmXFractionToMeters(0.5));
   rmAddObjectDefConstraint(fishID, fishVsFishID);
/*   rmAddObjectDefConstraint(fishID, rmCreateBoxConstraint("fish edge", rmXTilesToFraction(4), rmZTilesToFraction(4), 1.0-rmXTilesToFraction(4), 1.0-rmZTilesToFraction(4))); */
   rmAddObjectDefConstraint(fishID, rmCreateTerrainDistanceConstraint("fish land", "land", true, 10.0));
   
   rmPlaceObjectDefAtLoc(fishID, 0, 0.5, 0.5, 23*cNumberNonGaiaPlayers);


   // Player elevation.
   int avoidBuildings=rmCreateTypeDistanceConstraint("avoid buildings", "Building", 5.0);
   for(i=1; <cNumberPlayers)
   {
      int failCount=0;
      int num=rmRandInt(5, 10);
      for(j=0; <num)
      {
         int elevID=rmCreateArea("elev"+i+", "+j, rmAreaID("player"+i));
         rmSetAreaSize(elevID, rmAreaTilesToFraction(100), rmAreaTilesToFraction(500));
         rmSetAreaWarnFailure(elevID, false);
         rmAddAreaConstraint(elevID, avoidBuildings);
         rmAddAreaConstraint(elevID, avoidImpassableLand);
         if(rmRandFloat(0.0, 1.0)<0.3)
            rmSetAreaTerrainType(elevID, "sandb");
         rmSetAreaBaseHeight(elevID, rmRandFloat(3.0, 3.5));

         rmSetAreaMinBlobs(elevID, 1);
         rmSetAreaMaxBlobs(elevID, 5);
         rmSetAreaMinBlobDistance(elevID, 16.0);
         rmSetAreaMaxBlobDistance(elevID, 40.0);
         rmSetAreaCoherence(elevID, 0.0);

         if(rmBuildArea(elevID)==false)
         {
            // Stop trying once we fail 3 times in a row.
            failCount++;
            if(failCount==3)
               break;
         }
         else
            failCount=0;
      }
   }


   // Forest.
   int classForest=rmDefineClass("forest");
   int forestObjConstraint=rmCreateTypeDistanceConstraint("forest obj", "all", 6.0);
   int forestConstraint=rmCreateClassDistanceConstraint("forest v forest", rmClassID("forest"), 25.0);
   int forestSettleConstraint=rmCreateClassDistanceConstraint("forest settle", rmClassID("starting settlement"), 15.0);
   int forestTerrain=rmCreateTerrainDistanceConstraint("forest terrain", "Land", false, 3.0);

   // Player forests.   
   for(i=1; <cNumberPlayers)
   {
      failCount=0;
      int forestCount=rmRandInt(5, 8);
      for(j=0; <forestCount)
      {
         int forestID=rmCreateArea("player"+i+"forest"+j, rmAreaID("player"+i));
         rmSetAreaSize(forestID, rmAreaTilesToFraction(250), rmAreaTilesToFraction(300));
         rmSetAreaWarnFailure(forestID, false);
         rmSetAreaForestType(forestID, "palm forest");
         rmAddAreaConstraint(forestID, forestSettleConstraint);
         rmAddAreaConstraint(forestID, forestObjConstraint);
         rmAddAreaConstraint(forestID, forestConstraint);
         rmAddAreaConstraint(forestID, forestTerrain);
         rmAddAreaToClass(forestID, classForest);
      
         rmSetAreaMinBlobs(forestID, 8);
         rmSetAreaMaxBlobs(forestID, 15);
         rmSetAreaMinBlobDistance(forestID, 16.0);
         rmSetAreaMaxBlobDistance(forestID, 100.0);
         rmSetAreaCoherence(forestID, 0.2);

         // Hill trees?
         if(rmRandFloat(0.0, 1.0)<0.2)
            rmSetAreaBaseHeight(forestID, rmRandFloat(3.0, 4.0));

         if(rmBuildArea(forestID)==false)
         {
            // Stop trying once we fail 3 times in a row.
            failCount++;
            if(failCount==3)
               break;
         }
         else
            failCount=0;
      }
   }

   // Random island forests.
   int forestConstraint2=rmCreateClassDistanceConstraint("forest v forest2", rmClassID("forest"), 10.0);
   for(i=0; <bonusCount)
   {
      forestCount=rmRandInt(2, 3);
      for(j=0; <forestCount)
      {
         forestID=rmCreateArea("bonus"+i+"forest"+j, rmAreaID("bonus island"+i));
         rmSetAreaSize(forestID, rmAreaTilesToFraction(600), rmAreaTilesToFraction(800));
         rmSetAreaWarnFailure(forestID, false);
         rmSetAreaForestType(forestID, "palm forest");
         rmAddAreaConstraint(forestID, forestSettleConstraint);
         rmAddAreaConstraint(forestID, forestObjConstraint);
         rmAddAreaConstraint(forestID, forestConstraint2);
         rmAddAreaConstraint(forestID, forestTerrain);
         rmAddAreaToClass(forestID, classForest);
         rmSetAreaWarnFailure(forestID, false);
      
         rmSetAreaMinBlobs(forestID, 15);
         rmSetAreaMaxBlobs(forestID, 20);
         rmSetAreaMinBlobDistance(forestID, 16.0);
         rmSetAreaMaxBlobDistance(forestID, 500.0);
         rmSetAreaCoherence(forestID, 0.2);

         // Hill trees?
         if(rmRandFloat(0.0, 1.0)<0.2)
            rmSetAreaBaseHeight(forestID, rmRandFloat(2.0, 3.0));

         rmBuildArea(forestID);
      }
   }

   //rocks
   int rockID=rmCreateObjectDef("rock");
   rmAddObjectDefItem(rockID, "rock sandstone sprite", 1, 0.0);
   rmSetObjectDefMinDistance(rockID, 0.0);
   rmSetObjectDefMaxDistance(rockID, 600);
   rmAddObjectDefConstraint(rockID, avoidAll);
   rmAddObjectDefConstraint(rockID, avoidImpassableLand);
   rmPlaceObjectDefAtLoc(rockID, 0, 0.5, 0.5, 30*cNumberNonGaiaPlayers);

   // Bushes
   int bushID=rmCreateObjectDef("big bush patch");
   rmAddObjectDefItem(bushID, "bush", 4, 3.0);
   rmSetObjectDefMinDistance(bushID, 0.0);
   rmSetObjectDefMaxDistance(bushID, 600);
   rmAddObjectDefConstraint(bushID, avoidAll);
   rmPlaceObjectDefAtLoc(bushID, 0, 0.5, 0.5, 5*cNumberNonGaiaPlayers);


   int bush2ID=rmCreateObjectDef("small bush patch");
   rmAddObjectDefItem(bush2ID, "bush", 3, 2.0);
   rmAddObjectDefItem(bush2ID, "rock sandstone sprite", 1, 2.0);
   rmSetObjectDefMinDistance(bush2ID, 0.0);
   rmSetObjectDefMaxDistance(bush2ID, 600);
   rmAddObjectDefConstraint(bush2ID, avoidAll);
   rmPlaceObjectDefAtLoc(bush2ID, 0, 0.5, 0.5, 30*cNumberNonGaiaPlayers);

   int nearRiver=rmCreateTerrainMaxDistanceConstraint("near river", "water", true, 5.0);
   int riverBushID=rmCreateObjectDef("bushs by river");
   rmAddObjectDefItem(riverBushID, "bush", 3, 3.0);
   rmAddObjectDefItem(riverBushID, "grass", 7, 8.0);
   rmSetObjectDefMinDistance(riverBushID, 0.0);
   rmSetObjectDefMaxDistance(riverBushID, 600);
   rmAddObjectDefConstraint(riverBushID, avoidAll);
   rmAddObjectDefConstraint(riverBushID, nearRiver);
   rmPlaceObjectDefAtLoc(riverBushID, 0, 0.5, 0.5, 50*cNumberNonGaiaPlayers);

 // pyramid
   int pyramidsmallID=rmCreateObjectDef("pyramid");
   rmAddObjectDefItem(pyramidsmallID, "pyramid small", rmRandInt(1,1), 6.0);
   rmSetObjectDefMinDistance(pyramidsmallID, 0.0);
   rmSetObjectDefMaxDistance(pyramidsmallID, rmXFractionToMeters(0.5));
   rmAddObjectDefConstraint(pyramidsmallID, avoidAll);
   rmAddObjectDefConstraint(pyramidsmallID, avoidImpassableLand);
   rmAddObjectDefConstraint(pyramidsmallID, shortAvoidSettlement);
   rmAddObjectDefConstraint(pyramidsmallID, playerConstraint);
   rmAddObjectDefConstraint(pyramidsmallID, farAvoidSettlement);
   rmPlaceObjectDefAtLoc(pyramidsmallID, 0, 0.5, 0.5, 3*cNumberNonGaiaPlayers);

 // temples
   int templeID=rmCreateObjectDef("temple");
   rmAddObjectDefItem(templeID, "temple", rmRandInt(1,1), 6.0);
   rmSetObjectDefMinDistance(templeID, 0.0);
   rmSetObjectDefMaxDistance(templeID, rmXFractionToMeters(0.5));
   rmAddObjectDefConstraint(templeID, avoidAll);
   rmAddObjectDefConstraint(templeID, avoidImpassableLand);
   rmAddObjectDefConstraint(templeID, avoidBuildings);
   rmAddObjectDefConstraint(templeID, shortAvoidSettlement);
   rmAddObjectDefConstraint(templeID, playerConstraint);
   rmAddObjectDefConstraint(templeID, farAvoidSettlement);
   rmPlaceObjectDefAtLoc(templeID, 0, 0.5, 0.5, 2*cNumberNonGaiaPlayers);


   int columnID=rmCreateObjectDef("columns");
   rmAddObjectDefItem(columnID, "columns broken", rmRandInt(1,2), 3.0);
   rmSetObjectDefMinDistance(columnID, 0.0);
   rmSetObjectDefMaxDistance(columnID, rmXFractionToMeters(0.5));
   rmAddObjectDefConstraint(columnID, avoidAll);
   rmAddObjectDefConstraint(columnID, avoidBuildings);
   rmAddObjectDefConstraint(columnID, avoidImpassableLand);
   rmPlaceObjectDefAtLoc(columnID, 0, 0.5, 0.5, 3*cNumberNonGaiaPlayers);



   int baboonID=rmCreateObjectDef("lonely baboon");
   rmAddObjectDefItem(baboonID, "baboon", rmRandInt(1,2), 1.0);
   rmSetObjectDefMinDistance(baboonID, 0.0);
   rmSetObjectDefMaxDistance(baboonID, rmXFractionToMeters(0.5));
   rmAddObjectDefConstraint(baboonID, avoidAll);
   rmAddObjectDefConstraint(baboonID, avoidBuildings);
   rmAddObjectDefConstraint(baboonID, avoidImpassableLand);
   rmPlaceObjectDefAtLoc(baboonID, 0, 0.5, 0.5, cNumberNonGaiaPlayers);

   for(i=1; < cNumberNonGaiaPlayers)
   {
      int statueID=rmCreateObjectDef("statue"+i);
      if(rmRandFloat(0,1)<0.33)
         rmAddObjectDefItem(statueID, "statue pharaoh", 3, 200.0);
      else if(rmRandFloat(0,1)<0.5)
         rmAddObjectDefItem(statueID, "statue lion left", 3, 200.0);
      else
         rmAddObjectDefItem(statueID, "statue lion right", 3, 200.0);
      rmSetObjectDefMinDistance(statueID, 0.0);
      rmSetObjectDefMaxDistance(statueID, rmXFractionToMeters(0.5));
      rmAddObjectDefConstraint(statueID, avoidAll);
      rmAddObjectDefConstraint(statueID, avoidBuildings);
      rmAddObjectDefConstraint(statueID, avoidImpassableLand);
      rmPlaceObjectDefAtLoc(statueID, 0, 0.5, 0.5, 1);
   }

  
}

